@@ -17,9 +17,10 @@ import (
 	"github.com/golang/glog"
 	dputils "github.com/intel/sriov-network-device-plugin/pkg/utils"
 	"github.com/jaypipes/ghw"
-	sriovnetworkv1 "github.com/openshift/sriov-network-operator/pkg/apis/sriovnetwork/v1"
 	"github.com/vishvananda/netlink"
 	"k8s.io/apimachinery/pkg/util/wait"
+
+	sriovnetworkv1 "github.com/k8snetworkplumbingwg/sriov-network-operator/api/v1"
 )
 
 const (
@@ -30,9 +31,16 @@ const (
 	netClass              = 0x02
 	numVfsFile            = "sriov_numvfs"
 	scriptsPath           = "bindata/scripts/load-kmod.sh"
+	ClusterTypeOpenshift  = "openshift"
+	ClusterTypeKubernetes = "kubernetes"
 )
 
 var InitialState sriovnetworkv1.SriovNetworkNodeState
+var ClusterType string
+
+func init() {
+	ClusterType = os.Getenv("CLUSTER_TYPE")
+}
 
 func DiscoverSriovDevices() ([]sriovnetworkv1.InterfaceExt, error) {
 	glog.V(2).Info("DiscoverSriovDevices")
@@ -89,6 +97,9 @@ func DiscoverSriovDevices() ([]sriovnetworkv1.InterfaceExt, error) {
 		if dputils.IsSriovPF(device.Address) {
 			iface.TotalVfs = dputils.GetSriovVFcapacity(device.Address)
 			iface.NumVfs = dputils.GetVFconfigured(device.Address)
+			if iface.EswitchMode, err = GetNicSriovMode(device.Address); err != nil {
+				glog.Warningf("DiscoverSriovDevices(): unable to get device mode %+v %q", device.Address, err)
+			}
 			if dputils.SriovConfigured(device.Address) {
 				vfs, err := dputils.GetVFList(device.Address)
 				if err != nil {
@@ -107,6 +118,8 @@ func DiscoverSriovDevices() ([]sriovnetworkv1.InterfaceExt, error) {
 	return pfList, nil
 }
 
+// SyncNodeState Attempt to update the node state to match the desired state
+//
 func SyncNodeState(newState *sriovnetworkv1.SriovNetworkNodeState) error {
 	var err error
 	for _, ifaceStatus := range newState.Status.Interfaces {
@@ -114,6 +127,10 @@ func SyncNodeState(newState *sriovnetworkv1.SriovNetworkNodeState) error {
 		for _, iface := range newState.Spec.Interfaces {
 			if iface.PciAddress == ifaceStatus.PciAddress {
 				configured = true
+				if iface.EswitchMode == sriovnetworkv1.ESWITCHMODE_SWITCHDEV && ClusterType == ClusterTypeOpenshift {
+					// Skip sync for openshift, MCO will inject a systemd service to config switchdev devices.
+					break
+				}
 				if !needUpdate(&iface, &ifaceStatus) {
 					glog.V(2).Infof("syncNodeState(): no need update interface %s", iface.PciAddress)
 					break
@@ -125,7 +142,7 @@ func SyncNodeState(newState *sriovnetworkv1.SriovNetworkNodeState) error {
 				break
 			}
 		}
-		if !configured && ifaceStatus.NumVfs > 0 {
+		if !configured && ifaceStatus.NumVfs > 0 && ifaceStatus.EswitchMode != sriovnetworkv1.ESWITCHMODE_SWITCHDEV {
 			if err = resetSriovDevice(ifaceStatus); err != nil {
 				return err
 			}
@@ -594,3 +611,12 @@ func generateRandomGuid() net.HardwareAddr {
 
 	return guid
 }
+
+func GetNicSriovMode(pciAddress string) (string, error) {
+	glog.V(2).Infof("GetNicSriovMode(): device %s", pciAddress)
+	devLink, err := netlink.DevLinkGetDeviceByName("pci", pciAddress)
+	if err != nil {
+		return "", err
+	}
+	return devLink.Attrs.Eswitch.Mode, nil
+}
